package com.fawry.fawry_sdk

import android.app.Activity
import androidx.annotation.NonNull
import com.app.mylibrary.FawrySdk
import com.app.mylibrary.interfaces.FawryPreLaunch
import com.app.mylibrary.interfaces.FawrySdkCallbacks
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.EventChannel.EventSink
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result


/** FawrySdkPlugin */
class FawrySdkPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {

    private lateinit var channel: MethodChannel
    private var eventSink: EventSink? = null
    private lateinit var activity: Activity

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, CHANNEL_NAME)
        channel.setMethodCallHandler(this)

        // Prepare event channel
        EventChannel(flutterPluginBinding.binaryMessenger, EVENT_CHANNEL_NAME)
            .setStreamHandler(object : EventChannel.StreamHandler {
                override fun onListen(listener: Any?, event: EventSink) {
                    eventSink = event
                }

                override fun onCancel(listener: Any?) {
                    eventSink = null
                }
            })
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        if (call.method == METHOD_INIT) {

            // Get the args from Flutter.
            val launchJsonString = call.argument<String>(LAUNCH_MODEL_JSON_ARG)
            val baseUrl = call.argument<String>(BASE_URL_ARG)
            val lang: FawrySdk.Languages =
                if (call.argument<String>(LANGUAGE_ARG) == LANGUAGE_ARABIC)
                    FawrySdk.Languages.ARABIC
                else
                    FawrySdk.Languages.ENGLISH
            val launchModel = launchJsonString?.toFawryLaunchModel()

            if (baseUrl.isNullOrBlank()) {
                eventSink?.success(ResponseStatus(RESPONSE_ERROR, INVALID_BASE_URL).toJson())
                return
            }

            if (launchModel == null) {
                eventSink?.success(ResponseStatus(RESPONSE_ERROR, INVALID_LAUNCH_MODEL).toJson())
                return
            }

            if (::activity.isInitialized) {
                FawrySdk.launchAnonymousSDK(
                    activity,
                    lang,
                    baseUrl,
                    launchModel,
                    object : FawrySdkCallbacks {
                        override fun onPreLaunch(onPreLaunch: FawryPreLaunch) {
                            onPreLaunch.onContinue()
                        }

                        override fun onInit() {
                        }

                        override fun onSuccess(msg: String, data: Any?) {
                            eventSink?.success(
                                ResponseStatus(
                                    RESPONSE_SUCCESS,
                                    PAYMENT_SUCCEEDED,
                                    data.toString()
                                ).toJson()
                            )
                        }

                        override fun onFailure(error: String) {
                            eventSink?.success(
                                ResponseStatus(
                                    RESPONSE_ERROR,
                                    error,
                                    null
                                ).toJson()
                            )
                            eventSink?.error(CODE_PAYMENT_FAILED, error, error)
                        }

                        override fun onPaymentCompleted(msg: String, data: Any?) {
                            eventSink?.success(
                                ResponseStatus(
                                    RESPONSE_PAYMENT_COMPLETED,
                                    msg,
                                    data.toString()
                                ).toJson()
                            )
                        }
                    }
                )
            }
        } else {
            result.notImplemented()
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        this.activity = binding.activity
    }

    override fun onDetachedFromActivityForConfigChanges() {

    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {

    }

    override fun onDetachedFromActivity() {

    }

    companion object {
        private const val CHANNEL_NAME = "channel/fawry_sdk"
        private const val EVENT_CHANNEL_NAME = "event.channel/fawry_sdk"
        private const val METHOD_INIT = "init"
        private const val LAUNCH_MODEL_JSON_ARG = "launchModelJson"
        private const val BASE_URL_ARG = "baseURL"
        private const val LANGUAGE_ARG = "lang"

        private const val LANGUAGE_ARABIC = "ARABIC";
        private const val LANGUAGE_ENGLISH = "ENGLISH";

        private const val EVENT_PAYMENT_SUCCESS = "Payment Successful"
        private const val CODE_PAYMENT_FAILED = "1"

        private const val RESPONSE_SUCCESS = "100"
        private const val RESPONSE_ERROR = "101"
        private const val RESPONSE_PAYMENT_COMPLETED = "102"

        private const val INVALID_LAUNCH_MODEL = "Invalid LaunchModel"
        private const val INVALID_BASE_URL = "Invalid BASE_URL"
        private const val PAYMENT_SUCCEEDED = "Payment process succeeded"
    }
}
