import Flutter
import UIKit
import FawryFrameworkAnonymous


public class SwiftFawrySdkPlugin: NSObject, FlutterPlugin {
    
    private static var eventSink : FlutterEventSink!
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        
        let channel = FlutterMethodChannel(name: Constants.sdk_channel_name, binaryMessenger: registrar.messenger())
        let instance = SwiftFawrySdkPlugin()
        
        registrar.addMethodCallDelegate(instance, channel: channel)
        let eventChannel = FlutterEventChannel(name: Constants.eventChannel_callback, binaryMessenger: registrar.messenger())
        eventChannel.setStreamHandler(FawryStreamHandler())
        
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        
        if call.method == Constants.method_init {
            
            let args = call.arguments as! [String:Any?]
            
            guard let fawryModelString = args[Constants.launch_model_json_arg] as? String else {
                let responseStatus = ResponseStatus(status: Constants.error_code,
                                                    message: Constants.Messages.invalidLaunchModel,
                                                    data: nil)
                SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString)
                
                return
            }
            
            guard let baseURL = args[Constants.baseURL_arg] as? String else {
                let responseStatus = ResponseStatus(status: Constants.error_code, message: Constants.Messages.invalidBaseURL, data: nil)
                SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString)
                return
            }
            
            guard let dartLang = args[Constants.language_arg] as? String else {
                let responseStatus = ResponseStatus(status: Constants.error_code, message: Constants.Messages.invalidLanguage, data: nil)
                SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString)
                return
            }
            
            let decoder = JSONDecoder()
            guard let jsonData = fawryModelString.data(using: .utf8) else {
                let responseStatus = ResponseStatus(status: Constants.error_code, message: Constants.Messages.invalidLaunchModel, data: nil)
                SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString)
                return
            }
            
            do {
                let fawryLaunchModel = try decoder.decode(FawryLaunchModel.self, from: jsonData)
                let lang = (dartLang == Constants.dart_arabic_lang ? (AppLanguage.Arabic) : (AppLanguage.English))
                showFawrySDK(launchModel: fawryLaunchModel,baseURL: baseURL, lang: lang)
                
            } catch {
                
                let responseStatus = ResponseStatus(status: Constants.error_code, message: error.localizedDescription, data: nil)
                SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString)
                
            }
        }
    }
    
    func showFawrySDK (launchModel: FawryLaunchModel, baseURL: String, lang: String) {
        guard let fawryInstance = FrameworkHelper.shared else { return }
        guard let viewController = UIApplication.shared.delegate?.window??.rootViewController else { return }
        
        fawryInstance.launchAnonymousSDK(on: viewController,
                                         launchModel: launchModel,
                                         baseURL: baseURL,
                                         appLanguage: lang,
                                         enable3Ds: true,
                                         completionBlock: { (status) in
            guard let status = status else {return}
            let responseStatus = ResponseStatus(status: "\(status.rawValue)", message: Constants.Messages.paymentLoaded, data: nil)
            SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString ?? "")
            
        }, onPreCompletionHandler: { (error) in
            let responseStatus = ResponseStatus(status: Constants.error_code, message: error?.message ?? "", data: nil)
            SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString ?? "" )
        }, errorBlock: { (error) in
            let responseStatus = ResponseStatus(status: Constants.error_code , message: error?.message ?? "", data: nil)
            SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString ?? "")
        }, onPaymentCompletedHandler: {
            (response) in
            
            if let chargeResponse = (response) as? PaymentChargeResponse {
                let responseStatus = ResponseStatus(status: Constants.payment_completed_code , message: Constants.Messages.paymentCompleted, data: chargeResponse.convertToString)
                SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString ?? "")
            }
            
            if let errorResponse = response as? FawryError{
                let responseStatus = ResponseStatus(status: Constants.error_code , message: errorResponse.message ?? "", data: nil)
                SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString ?? "" )
            }
            
        }, onSuccessHandler: { (response) in
            guard let responseCharge = response as? PaymentChargeResponse else {
                return}
            let responseStatus = ResponseStatus(status: Constants.success_code , message: Constants.Messages.paymentSucceeded, data: responseCharge.convertToString)
            SwiftFawrySdkPlugin.eventSink(responseStatus.convertToString ?? "" )
            
        })
    }
    
    class FawryStreamHandler: NSObject, FlutterStreamHandler {
        public func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
            eventSink = events
            return nil
        }
        
        public func onCancel(withArguments arguments: Any?) -> FlutterError? {
            eventSink = nil
            return nil
        }
    }
    
}
