import 'dart:async';

import 'package:fawry_sdk/fawry_sdk_platform_interface.dart';
import 'package:fawry_sdk/model/fawry_launch_model.dart';
import 'package:flutter/services.dart';

class FawrySdk extends FawrySdkPlatformInterface {
  FawrySdk._();

  static FawrySdk instance = FawrySdk._();
  static const String _CODE_PAYMENT_FAILED = "1";
  static const String EVENT_CHANNEL_NAME = "event.channel/fawry_sdk";

  static const String LANGUAGE_ARABIC = "ARABIC";
  static const String LANGUAGE_ENGLISH = "ENGLISH";

  static const RESPONSE_SUCCESS = "100";
  static const RESPONSE_ERROR = "101";
  static const RESPONSE_PAYMENT_COMPLETED = "102";

  EventChannel _eventChannelCallbackResult = EventChannel(EVENT_CHANNEL_NAME);

  @override
  Future<bool> init(
      {required FawryLaunchModel launchModel,
      required String baseURL,
      required String lang}) async {
    try {
      return await FawrySdkPlatformInterface.instance
          .init(launchModel: launchModel, baseURL: baseURL, lang: lang);
    } on PlatformException catch (e) {
      if (e.code == _CODE_PAYMENT_FAILED) {
        throw "Payment Failed: $_CODE_PAYMENT_FAILED. Message: ${e.message}.";
      }
      throw "Unknown Error: Error ->  ${e.message}";
    } catch (e) {
      throw "Error Occurred: Message: $e";
    }
  }

  Stream callbackResultStream() {
    return _eventChannelCallbackResult.receiveBroadcastStream();
  }
}
